 {~ Displays an image from Santa Barbara ST4 CCD camera ~}

Program S4;
{program Show_CCD_ST4 by Jan Hollan using a unit from GRADA publishing house}

uses PG256,crt;
const cl=#13#10;
h1= cl+
'S4 <ST4-file name> [bottom level] [top level]'+cl
                                               +cl+
'shows a CCD frame from the ST4 camera: 192 columns and 165 rows of byte.'+cl
                                               +cl+
'Begins with a linear scale from 0 to 255, first <Enter> alters the scale'+cl;
h2=
'to a logarithmic one going from 10% to 99.99% of the distribution function.'+cl+
                                                                          cl+
'Bottom and top level may be given interactively as well, separated by a space.'+cl+
'Just the top (white) level may be changed by starting the line with a space.'+cl;
h3=
'A character as a first letter switches logarithmic/linear scale.'+cl
                                                                 +cl+
'Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1994,'+cl+
'(using a Turbo Pascal unit PG256 from the GRADA publishing house)'+cl;

var
 cc,k,m: byte;
 ch: char;
 intln: real;
 inp: file;
 Gray256_palette: RGB_Palette;
{ buf: array[1..192] of byte;}
 buf: array[1..166,1..192] of byte;
 distrib,Histogram: array[0..255] of word;
 NumRead,Wrong_pos: integer;
 aux_s:string;
 Bott_perc,Top_perc: word;
 Mean: byte;
const
 top: byte=255;
 bottom: byte=0;
 offs: word=$A000;
 Linear: boolean=false;

procedure help;
  begin
   write(h1); write(h2); write(h3)
  end;

procedure Stop;
begin
 PG256_end;
 TextColor(15);
 LowVideo; Write('Histogram of the file ');
 Highvideo; Write(paramstr(1));
 LowVideo;
 if length(paramstr(1))<12 then
  for k:=1 to 12-length(paramstr(1)) do write(' ');
 writeln(' exp./0.01s   fc.l./"  ap.ar/sq"   cal.f.');
 for k:=1 to 32 do write(' ');
 for k:=81 to 90 do write(chr(buf[166,k])); write('   ');
 for k:=91 to 99 do write(chr(buf[166,k])); write('  ');
 for k:=100 to 109 do write(chr(buf[166,k])); write('  ');
 for k:=110 to 119 do write(chr(buf[166,k])); write('  ');
 writeln;
 for k:=2 to 2+76 do write(chr(buf[166,k]));
 writeln;
 for k:=81+40 to 192 do if buf[166,k]>0 then write(chr(buf[166,k])) else write(' ');
 writeln;
 for k:=0 to 16 do
  begin
   write(k*15:3);
   for m:=0 to 14 do write(Histogram[15*k+m]:5);
   writeln
  end;
 write(255,Histogram[255]:5);
 for m:=1 to 14 do write('  +',m:2);

{ readln;
 write('   ');
 for m:=0 to 14 do write(m:5);
 writeln(cl);
 for k:=0 to 16 do
  begin
   write(k*15:3);
   for m:=0 to 14 do write(Distrib[15*k+m]:5);
   writeln
  end;
 writeln(255,Distrib[255]:5);}
 halt
end;

begin
 FillChar(Gray256_palette,3*256,0);
 FillChar(Histogram,512,0);
 If ParamCount>1 then
  begin
   val(ParamStr(2),bottom,Wrong_pos);
   if Wrong_pos > 0 then
    begin
     writeln('The second parameter is no bottom level.');
     help;
     halt
    end;
   If ParamCount>2 then
    begin
     val(ParamStr(3),top,Wrong_pos);
     if Wrong_pos > 0 then
      begin
       writeln('The third parameter is no top level.');
       help;
       halt
      end
    end
  end;

 intln:=(top-bottom)/62;
 if intln <= 0 then
  begin
   writeln('The top level is not greater than the bottom one.');
   halt
  end;

 if bottom>0 then for cc:=1 to bottom do with Gray256_palette[cc] do G:=0;
 for cc:=top to 255 do with Gray256_palette[cc] do G:=63;
 m:=bottom+1; k:=top-1;
 for cc:=m to k do with Gray256_palette[cc] do G:=1+trunc((cc-bottom)/intln);

{$I-}
 assign(inp,paramstr(1));
 reset(inp,192);
 if IORESULT<>0 THEN 
  begin 
   aux_s:=ParamStr(1);
   if not ((aux_s[1]='/') or (aux_s[1]='?')) then
    writeln('There is no file >',ParamStr(1),'< in this directory!');
   help;
   halt
  end;
{$I+}
 BlockRead(inp,buf,166,NumRead);
 close(inp);

 PG256_Start;
 Set256_Palette(Gray256_palette);

 for m:=1 to 165 do      {putting the image to the 320x200 screen}
  begin
   Move(buf[m,1], Mem[offs:0], 192);
   inc(offs,20);
   for k:=1 to 192 do inc(Histogram[buf[m,k]]);
  end;

  {scale of the gray scale}
 for cc:=0 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,cc);
 for cc:=0 to 25 do for k:=178 to 188 do Set256_Pixel(10*cc,k,255);
 for cc:=0 to 5 do for k:=189 to 199 do Set256_Pixel(50*cc,k,255);

 TextColor(200);
 GoToXY(26,1);
 Distrib:=Histogram;
 for m:=1 to 255 do inc(Distrib[m],Distrib[m-1]);

 readln(AUX_S);
 if aux_s='' then {The first <Enter> makes the scale logarithmic from 25% to 99.9%}
  begin
   Bott_perc:=trunc(165*192*0.25);
   Top_perc:=trunc(165*192*0.9999);
   m:=0;
   while bottom=0 do
    begin
     if Distrib[m]>Bott_perc then Bottom:=m;
     inc(m)
    end;
   while (top=255) and (m<254) do
    begin
     inc(m);
     if Distrib[m]> Top_perc then Top:=m
    end
  end
 else
  if ord(aux_s[1])<32 then Stop;

repeat
 if aux_s<>'' then
  begin
   val(aux_s,m,Wrong_pos);
   if Pos(' ',Aux_s)>0 then
    begin
     if Pos(' ',Aux_s)>1 then
      begin
       val(copy(aux_s,1,pos(' ',Aux_s)-1),bottom,Wrong_pos);
       if Wrong_pos>0 then Stop
      end;
     val(copy(aux_s,pos(' ',Aux_s)+1,length(aux_s)-pos(' ',Aux_s)),top,Wrong_pos);
     if Wrong_pos>0 then Stop
    end
   else bottom:=m
  end;

 if Top <= bottom then Stop;

 if bottom>0 then for cc:=1 to bottom do with Gray256_palette[cc] do G:=0;
 for cc:=top to 255 do with Gray256_palette[cc] do G:=63;

 Mean:=(top+6*bottom) div 7;
  {to prevent a too long final white step that would blind the observer:}
 if top<230 then
  begin
   for cc:=top+2 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,Mean);
   for cc:=0 to top+1 do for k:=167 to 177 do Set256_Pixel(cc,k,cc)
  end
 else
   for cc:=0 to 255 do for k:=167 to 177 do Set256_Pixel(cc,k,cc);

 m:=bottom+1; k:=top-1;

 if Linear then
  begin
   intln:=(top-bottom)/62;
   for cc:=m to k do with Gray256_palette[cc] do
    G:=1+trunc((cc-bottom)/intln)
  end
 else
  begin
   IntLn:=ln(top-bottom)/62;
   for cc:=m to k do with Gray256_palette[cc] do
    G:=1+Trunc(ln(1+cc-bottom)/IntLn)
  end;

 Set256_Palette(Gray256_palette);
 GoToXY(26,WhereY);
 read(ch);
 if ord(ch)<32 then Stop;
 readln(aux_s);
 if (ord(ch) in [33 .. 47]) or (ord(ch)>57) then Linear:= not Linear
 else aux_s:=ch + aux_s;
until false

end.
