{~ Motion of Galileian satellites, by Rudolf Novak ~}

program Vypocet_poloh_jupiterovych_mesicu;
          {parameters decoding and minor corrections by Jan Hollan}

uses dos,crt,graph,
     solar_ut,planets,params,un_str_m,str_num,angles_o;
const
h1=cl+
'A program from Rudolf Novak, N.Copernicus Obs. and Planetarium in Brno, 1996,'+cl+
' (subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl+cl;
h1a=
'shows positions of Galilean satellites:'+cl+
   +cl+
'Jup dDD[.MM[.YY]] tHH[.MM[.SS]] UT# C'+cl+cl;
h2=
'D and T are initially taken from the system date and time'+cl+
'UT is the t-UTC difference / 1 h (default 1 for Nov. to March, else 2)'+cl+
'C Output just coordinates (as an input for subsequent programme).'+cl;
h3=
  cl+
'No parameters are needed - the current moment is the start then.'+cl;

just_coord:boolean=false;
sx1o:integer=0;
sx2o:integer=0;
sx3o:integer=0;
sx4o:integer=0;

var
    ch :char;
    st:integer;
    Name:string;
    grmode,i,grdriver,sx1,sx2,sx3,sx4:integer;
    l,b,r,lg,bg,rg:real;

procedure Help;
begin
 write(h1,h1a,h2,h3,hTT); halt
end;

function arcsin(x:real):real;
begin
arcsin:=arctan(x/sqrt(1-sqr(x)))*180/pi;
end;

function asin(x:real):real;
begin
asin:=sin(x*pi/180);
end;

function acos(x:real):real;
begin
acos:=cos(x*pi/180);
end;

procedure Vypocet;         (*Toto je proc. ops. z algor.*)
var
    A1,B,p,mz,mj,lzj,
    sjd,delta,psi,mi1,mi2,mi3,mi4,x1,x2,x3,x4  : real;
begin
     sjd:=(jd-2415020);
     mz:=(358.476 + (0.9856003*sjd));
     mj:=(225.328 + (0.0830853*sjd));
    Lzj:=(221.647 + (0.9025179*sjd));
     mz:=Bas_Int(mz,360);
     mj:=Bas_Int(mj,360);
     lzj:=Bas_Int(lzj,360);
     A1:=(1.92*asin(mz))+(0.02*asin(2*mz));
     B:=(5.537*asin(mj))+(0.167*asin(2*mj));
     P:=lzj+A1-B;
     delta:=sqrt(28.07-(10.406*acos(p)));
     psi:=arcsin(asin(P)/delta);
     mi1:=84.5506+203.4058630*(sjd-delta/173)+psi-b;
     mi2:=41.5015+101.2916323*(sjd-delta/173)+psi-b;
     mi3:=109.9770+50.2345169*(sjd-delta/173)+psi-b;
     mi4:=176.3586+21.4879802*(sjd-delta/173)+psi-b;
     x1:=5.906*asin(mi1);
     x2:=9.397*asin(mi2);
     x3:=14.989*asin(mi3);
     x4:=26.364*asin(mi4);
    sx1:=round(270+(X1*10));
    sx2:=round(270+(X2*10));
    sx3:=round(270+(X3*10));
    sx4:=round(270+(X4*10));
end;

procedure kresli;
  begin
     grdriver:=0;
     grmode:=1;
     initgraph(grdriver,grmode,getenv('bgi'));
  end;

procedure plot_sat(sxn:integer;var sxo:integer);
begin
 if (sxn < 260) or (sxn > 280) or (sxn>sxo) then
  circle(sxn,210,1);
 sxo:=sxn;
end;


begin  {of the MAIN PROGRAMME}
FileMode:=0; {will open the files by reset as Read Only
               -- they can be then marked so even if not on a CD;
               here it enables to have *.bgi and *.chr read-only}
  if paramcount>0 then for i:= 1 to paramcount do
   begin
    Par:=paramstr(i);
    Par:=__cvtstr(Par,_To_UpCase_Str);
    if (Par[1]='/') or (Par[1]='-') then
      Par:=copy(Par,2,length(par)-1);
    case Par[1] of
      '?','-': Help; {- added to respect unix --help even without the help}
      'C': if length(Par)=1 then just_coord:=true;
      'D': if not Par_Date then halt;
      'F': if not Par_Fi then halt;
      'L': if not Par_Lambda then halt;
      'T': if not Par_Time_or_TT then halt;
      'U': if not Par_UT then halt;
    end; {of case}
   end; {of for - of processing parameters}
  Get_JD;
  if not tt_UT_set then tt_ut:=tt_UTC_in_days(JD);
  if not t_UTC_given then
   if MM in [4..10] then t_UT:=2 else t_UT:=1;
  if Just_coord then
   begin
    Planet_hel(5,jd+tt_ut,l,b,r,Name);
    Hel_to_Geo(jd+tt_ut,l,b,r,lg,bg,rg);
    LB_RD(lg,bg,l,b);
    writeln('RA',l:4:3,' De',b:4:3);
    halt
   end;
     kresli;
     cleardevice;
     {settextstyle(smallfont,horizdir,5);}
     outtextxy(5,10,' Day:   ');
     outtextxy(5,20,'Month: ');
     outtextxy(getmaxx-200,30,'ESC = End');
     outtextxy(getmaxx-200,50,' +  = Change step');
     outtextxy(getmaxx-200,getmaxy-120,'1 .. Io');
     outtextxy(getmaxx-200,getmaxy-100,'2 .. Europa');
     outtextxy(getmaxx-200,getmaxy-80,'3 .. Ganymedes');
     outtextxy(getmaxx-200,getmaxy-60,'4 .. Kallisto');
     line(1,1,getmaxx-1,1);
     line(1,1,1,getmaxy-1);
     line(getmaxx-1,getmaxy-1,1,getmaxy-1);
     line(getmaxx-1,getmaxy-1,getmaxx-1,1);
     ch:=#0;
     st:=1;
     i:=0;
     line(1,getmaxy-40,getmaxx-1,getmaxy-40);
     setfillstyle(1,8);
     bar(2,getmaxy-39,getmaxx-3,getmaxy-2);
     outtextxy(50,getmaxy-30,'Jupiter 1.1 - Rudolf K. Novak, N. Copernicus Observatory in Brno, 1996');
     outtextxy(50,getmaxy-80,'[North Up --- West Right]');
     outtextxy(50,getmaxy-60,'Press space-bar to run or pause animation');
    repeat
     inc(i);
     vypocet;
     setfillstyle(1,0);
     bar(2,110,630,190);
     bar(45,10,63,30);
     bar(2,190,630,263);
     setcolor(15);
     setfillstyle(1,15);
     circle(270,210,10);
     setfillstyle(1,15);
     floodfill(270,210,15);
     Name:=DoW(JD,t_UT,_English);
     setcolor(7);
     outtextxy(50,10,SI(3,nd)+' '+copy(Time,1,5));
     outtextxy(50,20,SI(4,MM));
     plot_sat(sx1,sx1o);
     plot_sat(sx2,sx2o);
     plot_sat(sx3,sx3o);
     plot_sat(sx4,sx4o);
     outtextxy(sx1,150,'1');
     outtextxy(sx2,150,'2');
     outtextxy(sx3,150,'3');
     outtextxy(sx4,150,'4');
     if keypressed then
      begin
        ch:=readkey;
        if ch =#43 then
         begin
          outtextxy(getmaxx-240,84,'New step (minutes): ');
          gotoxy(70,6);
          read(st);
          setfillstyle(1,0);
          bar(getmaxx-250,70,600,100);
         end;
        if ch=#32 then ch:=readkey;
      end;
     JD:=JD+st/60/24;
     if i=1 then ch:=readkey;
     delay(150);
     setcolor(0);
     outtextxy(50,10,SI(3,nd)+' '+copy(Time,1,5));
     outtextxy(50,20,SI(4,MM));
    until ch=#27;
    closegraph;
end.