 {~ select fields from, e.g., tyc_main.dat and writes them into a lesser file ~}

program tyc_sel;
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

uses DbBridge, dos, str_num;

Procedure help;
 const cl=#13#10;
 begin
  writeln(cl,
'Tyc_sel [i<index file>] [n<new indices output file>]',cl,
'        [d<original data file>] [o<output data file>]',cl,
'selects fields from original file and writes them into a lesser file.',cl,cl,

'i  is a file saying, which intervals are to be chosen (default tyc_sel.int).',cl,
' It has to contain in the columns 2 to 4 and 6 to 8 starts and ends of',cl,
' the wanted fields. Start can be empty, if same as the end.',cl,
' Non-conforming lines are ignored.',cl,
'n  will give those intervals for the new data file (default tyc_sel.new)',cl,
' (its contents may be then inserted as a column into the i-file by a suitable',cl,
'  editor)',cl,
'd  is the original data file (default tyc_main.dat)',cl,
'o  is the output data file (default tyc_sel.dat)',cl,cl,

'(The same job could be probably made by some standard',cl,
' text-manipulation programme.)',cl,cl,

'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'
         );
  halt;
 end;

const
 reclen=352;
 NuField:byte=0;
 startnew:word=1;
 NoR:longint=0;
 intfileName:pathstr='tyc_sel.int';
 fcatName:pathstr='tyc_main.dat';
 NewintfileName:pathstr='tyc_sel.new';
 NewfileName:pathstr='tyc_sel.dat';

var
 recCat:array[1..451] of char;
 i,j:byte;
 linestr,par:string;
 startb,endb:word;
 answ:char;

type Finforec = record
                len   : byte;
                start : word;
             newstart : word;
               end;
var
 FInfo:array[1..50] of Finforec;
 fcat,Newfile:file;
 intfile:text;

function CatRead(NofRec:longint;var reco:DbStr):boolean;
 var i:byte;
 begin
   Seek(fcat,NofRec);
   if eof(fcat) then begin CatRead:=false; exit end
   else CatRead:=true;
   BlockRead(fcat,recCat,1);
     for i:=1 to NuField do
      begin
       move(recCat[FInfo[i].start],reco[i][1],FInfo[i].len);
       reco[i][0]:=chr(FInfo[i].len);
     end;
 end;

begin
 i:=paramcount;
 if i>0 then
  for j:=1 to i do
   begin
    par:=paramstr(j);
    if par[1] in ['/','-'] then par:=copy(par,2,length(par)-1);
    case par[1] of
     '?','h': help;
     'i' : intfileName:=copy(par,2,length(par)-1);
     'd' : fcatName:=copy(par,2,length(par)-1);
     'n' : NewintfileName:=copy(par,2,length(par)-1);
     'o' : NewfileName:=copy(par,2,length(par)-1);
    end;
   end;
 if FSearch(intfileName,'')='' then help;
 assign(intfile,intfileName);
 reset(intfile);
 while not eof(intfile) do
  begin
   readln(intfile,linestr);
   if linestr[1] = ' ' then {not a comment line}
    begin
     startb:=I_S(copy(linestr,2,3));
     endb:=I_S(copy(linestr,6,3));
     if not ((startb=0) and (endb=0)) then {perhaps a valid field line}
      begin
       inc(NuField);
       if startb=0 then
        begin
         FInfo[NuField].len:=1;
         FInfo[NuField].start:=endb;
        end
       else
        begin
         FInfo[NuField].len:=1+endb-startb;
         FInfo[NuField].start:=startb;
        end;
       FInfo[NuField].newstart:=startnew;
       inc(startnew,FInfo[NuField].len);
      end
    end
  end;
 close(intfile);

 if FSearch(intfileName,'')<>'' then
  begin
   writeln(intfileName,' exists. Overwrite? (Y/n)');
   readln(answ);
   if upcase(answ)='N' then help;
  end;
 assign(intfile,NewintfileName);
 rewrite(intfile);
 write(intfile,'New intervals of fields selected by ',intfileName,' are:',cl);
 startb:=1;
 for i:=1 to NuField do
  begin
   endb:=startb-1+FInfo[i].len;
   writeln(intfile,startb:4,'-',endb:3);
   startb:=endb+1;
  end;
 close(intfile);

 if FSearch('tyc_main.dat','')='' then help;
 assign(fcat,'tyc_main.dat');
 reset(fcat,reclen);
 if FSearch(NewfileName,'')<>'' then
  begin
   writeln(NewfileName,' exists. Overwrite? (Y/n)');
   readln(answ);
   if upcase(answ)='N' then help;
  end;
 assign(newfile,NewfileName);
 rewrite(newfile,startb);
 while CatRead(NoR,rec) do
  begin
   for i:=1 to NuField do
    move(rec[i][1],recCat[FInfo[i].newstart],FInfo[i].len);
   recCat[startb]:=#10;
   BlockWrite(newfile,recCat,1);
   inc(NoR);
  end;
 close(fcat);
 close(Newfile);
end.
