 {~ Unit used just by the Map_mGSC program to sort and plot stars ~}

unit map_mg_p; {by J. Hollan}

 {simplified and modified from a similar unit of Map_BSC programme}

interface

uses graph,
     graph_m8,solar_ut,DbBridge,un_str_m,str_num;

const
 Faintness_Desc:byte=200;
 Faintness_base:integer=50;   {dmag}
 ToFont_dmag:byte = 10;
var
 Faintness_str : string[3];
 CenX,CenY,RigX:word;

function VT(Vdmag:integer):integer; {:=transformed V faintness / 1 dmag =
                                     =(Vcmag-Faintness_base) div ToFont_dmag}
function VdmagF2R(VdmagF:integer):word;
               {diameter of star image from transformed V faintness / 1 dmag}
function V2R(Vdmag:integer):word;
                      {diameter of star image from true V faintness / 1 dmag}
procedure process_ps_star(XRA,YDec:longint;Vtrue:integer);
procedure plot_rest;

implementation
type
 poi_st=^star;
 star=record
    Vs: integer; {(faintness of star)/(1 dmag)}
    r: word; {radius of star's circle}
    xc,yc:longint;    {cartesian coordinates}
    p_up, p_down:poi_st;  {pointers to following and preceding star}
   end;
var
 i_star, {index of a star in the queue}
 DistC,  {distance of centres of two stars,}
 SumR    {sum of their radii + white space between them}
      : word;
 n_further: integer; {number of further stars to be checked for overlap}
const
 White_space: byte=10;  {white space between two stars in PS file}
 n_queue: word=0;       {number of stars in the queue}
 p_star: poi_st=nil;    {a star within the examined queue}
 p_near: poi_st=nil;    {star which is overlaped by a new brighter one}
 p_new: poi_st=nil;     {new star within a map}
{ p_sum: poi_st=nil;}     {a fixed star which is being modified}
 p_top :poi_st=nil;     {top of queue - last added star mostly}
 p_bott:poi_st=nil;     {bottom of queue}
 p_plot: poi_st=nil;    {record outside queue to be plotted and cleared}
 d_star: poi_st=nil;    {desription queue}
 nm_queue:word=1200;    {maximum size  of the whole queue}
 NoPlotted: word=0;     {counter of plotted stars}
 times_r: word=6;       {number of queue members to be checked for overlap =
                        = times_r * r_of_star}

function VT(Vdmag:integer):integer; {:=transformed V faintness / 1 dmag}
begin
 VT:=(10*(Vdmag-Faintness_base)) div ToFont_dmag;
        {ToFont_dmag larger than 10 makes the scale less steep,
                    smaller than 10 makes it steeper than in reality,
     default value of 10 makes areas of circles proportional to brightness}
end;

function VdmagF2R(VdmagF:integer):word;
 {diameter of star image from transformed V faintness / 1 dmag}
var cn:byte;
begin
 cn:=gm_cn_f(VdmagF);
 if star_radius[cn]=0 then
  star_radius[cn]:=gm_star_font_radius(cn);
 VdmagF2R:=star_radius[cn];
end;

function V2R(Vdmag:integer):word;
 {diameter of star image from true V faintness / 1 cmag}
begin
 V2R:=VdmagF2R(VT(Vdmag));
end;

procedure is_near_and_further;
begin
 if (SumR>DistC) and (p_star^.r<p_new^.r) then p_near:=p_star;
 p_star:=p_star^.p_down;
 inc(i_star);
end;

procedure look_through_the_queue;
begin
 i_star:=1;
 p_star:=p_top;
 p_near:=nil;
     {possible overlaps are to be sought for
      and then the star is to be placed into the queue }
   if (p_star<>nil) then
    begin
     i_star:=0;
     n_further:=times_r*p_new^.r;
     if n_further>0 then     {seeking possible overlaps}
      begin
       p_star:=p_star^.p_down;
       while (p_star<>nil) and (i_star<n_further) do
                {using loose but quick criterion}
         begin
          DistC:=
           ((abs(p_new^.xc-p_star^.xc)+abs(p_new^.yc-p_star^.yc))*7) div 10;
          SumR:=p_new^.r + p_star^.r + White_space;
          is_near_and_further;
         end
      end
    end;

   with p_new^ do
    if p_near=nil then {it is placed to the top}
     begin
      p_down:=p_top;
      p_top^.p_up:=p_new;
      p_top:=p_new
     end
    else  {the new star is placed before a near fainter one}
     begin
      p_up:=p_near;
      p_down:=p_near^.p_down;
      p_near^.p_down:=p_new;
      if p_down=nil then   {new star got to the bottom}
       p_bott:=p_new
      else
       p_down^.p_up:=p_new;
     end
end;

procedure plot_star;
begin
 with p_plot^ do
  gm_star(xc,yc,VT(Vs),r,0,false,'');
 if p_plot^.Vs<=Faintness_desc then
  begin  {faintness in dmag right from the star}
   p_plot^.p_down:=d_star;
   d_star:=p_plot;
  end
 else
  Dispose(p_plot);
 inc(NoPlotted);
end;

procedure take_star_and_plot;
begin
 p_star:=p_bott;
 dec(n_queue);
 p_plot:=p_star;
 p_star:=p_star^.p_up;
 p_bott:=p_star;
 if p_star<> nil then
  p_star^.p_down:=nil;
 plot_star;
end; {of plotting a star}


procedure process_ps_star(XRA,YDec:longint;Vtrue:integer);
begin
     New(p_new);     {filling a new record of a star}
     p_near:=nil;
     with p_new^ do
      begin
       Vs:=Vtrue;
       r:=V2R(Vtrue);
       xc:=xRA;
       yc:=yDec;
       p_down:=nil;
       p_up:=nil;
       inc(n_queue);
      end;

     if n_queue>1 then  {looking in the first part of the queue for
                  the same fixed stars and stars which would be overlapped}
      look_through_the_queue
     else              {the very first star, n_queue=1}
      begin
       p_bott:=p_new;
       p_top:=p_new;
      end;

     if n_queue>nm_queue then
       take_star_and_plot;
end;

procedure plot_rest;
begin
 repeat take_star_and_plot until p_bott=nil;
 gm_verbatim('gsave 0.5 setgray'+#13#10);
 gm_comm(#13#10+'% center cross once more:');
 gm_line(CenX -192,CenY,CenX +192,CenY);
 gm_line(CenX, CenY -192, CenX, CenY +192);
 if d_star<>nil then
  begin
   gm_comm(#13#10+'% Explicit faintnesses / 1 dmag:');
   gm_verbatim('0 setgray'+#13#10);
   while d_star<>nil do
    begin
     with d_star^ do
      begin
       str(Vs,Faintness_str);
       if xc + r + 48 + TextWidth(Faintness_str) < RigX
        then
         gm_outxtxy(xc,yc,'',Faintness_str,'','l',0, r + 16,0)
       else
         gm_outxtxy(xc,yc,'',Faintness_str,'','r',0,-r - 16,0);
      end;
     p_plot:=d_star;
     d_star:=d_star^.p_down;
     Dispose(p_plot);
    end;
  end;
 gm_verbatim('0.5 setgray'+#13#10);
 gm_comm(#13#10+'% center cross once more:');
 gm_line(CenX -192,CenY,CenX +192,CenY);
 gm_line(CenX, CenY -192, CenX, CenY +192);
 gm_verbatim('grestore'+#13#10);
end;

end.
