 {~ Angular distance of two points (or stars in the Bright Stars Catalogue) ~}

Program Ang_Dist; {by Jan Hollan; uses B5_int.dbf}
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

uses DbBridge, angles_o, Un_Str_M, str_num, solar_ut, params, precesse,
 bsc_dir;      {just the loc_bsc constants}

const
h1=
'PADi <point 1> <point 2>'+cl+cl+
'computes position angle and angular distance of the second point'+cl+
'with respect to the first one.'+cl+cl;
h2=
'The point can be given through its HR number (in the Bright Stars Catalogue)'+cl+
' as HR#,'+cl+
'by its Greek/Flamsteed code preceded by N as nAlp_Cas, nAlp1Sco, n42_Eri,'+cl;
h3=
'or by giving RA# (in degrees) or RAhh:mm:ss and'+cl+
'             De#   -  "  -    or DEdd:mm:"" .'+cl+cl;
h3a=
'More than two points can be given, then a matrix of for all pairs'+cl+
'will be computed.'+cl2;
h3b=
' IR asks for output of angular distance just in milliradians'+cl+
' ID        -            "            -          degrees'+cl+
' IM        -            "            -          minutes'+cl2;
h3c=
' R[-] asks for (not)including refraction; if yes, than the following'+cl+
'    parameters are of importance:'+cl2;
h4=
' D  is date (initial value is the current one)'+cl+
' T  is time (default is the current one)'+cl+
' UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)'+cl+
' L  is the Longitude of the observing place, default that of '
 +MyLocation_Name+','+cl;
h5=
' F  is the latitude      -               "                -  '+cl+
' E   equinox year (default 2000.0, i.e. J2000), for computing precession '+cl;
h7=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

ns: byte=1;
ra: real=-999;
de: real=-999;
BSC:boolean=false;
part_given:boolean=false;
Greek_given:boolean=false;
AllVisible: boolean=true;
Refract:boolean=false;
all_units: boolean=true;
in_units:char='D';
PrecDate:real=2000.0;

var
hr,hrhh,hrmm,hrss,tt: word;
dd:integer;
na,ndec,ParN:byte;
pa,ad:real;
ras,des: array[1..7] of real;
star_des: array[1..7] of string[15];
star_des_b: array[1..56] of byte absolute star_des;
j,k:integer;

ParVal:string;
nflam:integer;
nname:byte;
sname,cname:string[3]; pname:char;
found:boolean;

procedure Help(s:string);
begin
 if BSC then DbClose;
 if s<>'' then writeln(cl+s+cl);
 write(h1,h2,h3);
   c_o_halt;
   write(h3a,h3b,h3c,h0enterlast);
   cont_or_halt;
   write(h0clear,h4,h5,cl,h_refraction,cl,h_dir_bsc,cl,h7);
 halt;
end;

procedure one_star;
begin
 ras[ns]:=ra;
 des[ns]:=de;
 ra:=-999;
 de:=-999;
 if star_des[ns]='' then
  star_des[ns]:='['+SRe(6,2,ra)+';'+SRe(6,2,de)+']';
 inc(ns);
end;

begin
FillChar(star_des_b[1],56,0);
if paramcount=0 then help('')
else
 for ParN:= 1 to paramcount do
  begin
   Par:=ParamStr(ParN);
   if Par[1] in ['/','-'] then Par:=copy(Par,2,length(par)-1);
   if length(par)>1 then ParVal:=copy(par,2,length(par)-1);
   Par:=__cvtstr(Par,_To_UpCase_Str);
   case Par[1] of
    '?','-': Help(''); {- added to respect unix --help even without the help}
    'H','N':
        if length(par)>1 then
         begin
          if not BSC then
            if not DbInitSearch(Loc_bsc,'b5_int.dbf') then
             help(
'The Bright Star Catalogue not found in current directory or directories:'+cl+
Loc_bsc)
           else
            begin
             BSC:=true;
             na:= DbFieldNum('rh2');
             ndec:=DbFieldNum('dd2');
             nname:=DbFieldNum('name');
            end;
          hr:=1;
          if par[1]='H' then
           begin
            hr:=round(Par_num(copy(Par,2,length(par)-1)));
            if not DbRead(hr,rec) then
             help('There is no such number in the Bright Star Catalogue: '
                   +SI(4,HR));
           end
          else if length(par)>6 then
           begin
            found:=false;
            if pos('_',ParVal)>0 then
             begin
              sname:=ItemStrD('_',1,ParVal);
              nflam:=I_S(sname);
              Greek_given:= nflam=0;
              cname:=ItemStrD('_',2,ParVal);
              pname:='_';
              part_given:=false;
             end
            else
             begin
              sname:=copy(ParVal,1,3);
              nflam:=I_S(sname); {nonsense for Flamsteed number<100}
              Greek_given:= nflam=0;
              cname:=copy(ParVal,5,3);
              pname:=ParVal[4];
              part_given:=true;
             end;
            hr:=1;
            if not DbRead(hr,rec) then;
            repeat
             if copy (rec[nname],8,3)=cname then
              if Greek_given then
               begin
                found:=copy(rec[nname],4,3)=sname;
		if found then 
                 if part_given then found:= found and 
		                           (pname=rec[nname][7]);
               end
              else
               begin
                found:=I_S(copy(rec[nname],1,3))=nflam;
                if part_given then found:= found and (rec[nname][7]=pname);
               end;
             inc(hr);
            until found or not DbRead(hr,rec);
            if not found then help(+sname+pname+cname+'  not found');
           end;
          DbDisp(hr);
          writeln;
          val(__cvtstr(rec[na],_rem_white_str),hh,j);
          val(__cvtstr(rec[na+1],_rem_white_str),mm,j);
          val(__cvtstr(rec[na+2],_rem_white_str),ss,j);
          tt:=ss-(ss div 10)*10;
          ss:=ss div 10;
          if Bab2Dec(1,hh,mm,ss,tt,ra,'H') then;
          ra:=ra*15;
          val(__cvtstr(rec[ndec],_rem_white_str),dd,j);
          val(__cvtstr(rec[ndec+1],_rem_white_str),mm,j);
          val(__cvtstr(rec[ndec+2],_rem_white_str),ss,j);
          tt:=ss-(ss div 10)*10;
          ss:=ss div 10;
          if pos('-',rec[ndec])>0 then
           begin
            dd:=abs(dd);
            if Bab2Dec(-1,dd,mm,ss,tt,de,'D') then
           end
          else
            if Bab2Dec(1,dd,mm,ss,tt,de,'D') then;
          star_des[ns]:=par;
          one_star;
         end
        else help('');
    'I': if length(par)>1 then
          begin
           in_units:=par[2];
           all_units:=false;
          end;
    'R': if length(par)>2 then
          case par[2] of
          'A':
            begin
             par:=copy(par,3,length(par)-2);
             val(par,ra,j);
             if j > 0 then
              if BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'H') then
               ra:=ra*15
              else
               if not BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'D') then
                help(
   'RA gives no valid right ascension (either degrees or hh:mm:ss): '+par);
             if de<>-999 then one_star;
            end;
          'H':
            begin
             Relative_Humidity:=ss2r(par,3);
             if Relative_Humidity<0.01 then
              help('RH gives no valid relative humidity: '+par);
            end;
           '-': Refract:=false;
          end
         else
          Refract:=true;
   'D': if length(par)>1 then
          if par[2]='E' then
           begin
            if length(par)>2 then
             begin
              par:=copy(par,3,length(par)-2);
              val(par,de,j);
              if j > 0 then
               if BabS2Dec(Par,hrhh,hrmm,hrss,tt,decl,'D') then
               else
                help(
   'De gives no valid declination (either degrees or dd:mm:ss): '+par);
             end
            else
                help(
   'De gives no declination (either degrees or dd:mm:ss): '+par);
            if ra<>-999 then one_star;
           end
          else if not Par_Date then help('');
   'P': if length(par)>2 then
          case par[2] of
          'R':
            begin
             pressure_db_101325:=ss2r(par,3);
             if pressure_db_101325<0.02 then
              help('PR gives no valid pressure divided by 1013.25 Pa: '+par);
            end
          else
           help(par+' is no known parameter!')
          end
        else
         help(par+' gives no value!');
   'C': if length(par)>2 then
          if par[2]='T' then
           begin
            Celsius_temp:=ss2r(par,3);
            if Celsius_temp<-273 then
             help('CT gives no valid Celsius temperature / 1 K: '+par);
           end
          else
           help(par+' is no known parameter!')
        else
         help(par+' gives no value!');
   'E': PrecDate:=ss2r(par,3);
   'F': if not Par_Fi then help('');
   'L': if length(Par)>2 then
         case  Par[2] of
          'B': Loc_BSC:=copy(Par,3,length(par)-2);
         {'E': Loc_obj:=copy(Par,3,length(par)-2);}
         else
          if not Par_Lambda then help(' ');
         end
        else
         if not Par_Lambda then help(' ');
   'T': if not Par_Time then help('');
   'U': if not Par_UT then help('');
   else help('');
   end;
  end;

if BSC then DbClose;

dec(ns);
if ns<2 then help('Nothing to do - two points needed to make anything');

if refract then
 begin
  Get_JD;
  if PrecInit(PrecDate,YYYYreal(JD)) then;
  for j:=1 to ns do
   begin
    Precess(ras[j],des[j]);
    A_h_PA(SM,ras[j],des[j],A,h,PA,Air_mass);
    if h>-0.8 then
     h:=refract_d(h,pressure_db_101325,Celsius_temp,Relative_humidity)
    else
     AllVisible:=false;
    ras[j]:=-A; des[j]:=h;
   end;
 end;

for j:=1 to ns-1 do
 for k:=j+1 to ns do
  begin
   ang_dist_posi_angD2(ras[j],des[j],0,90,ras[k],des[k],pa,ad);
   if all_units then
    begin
if ns>2 then writeln('Stars:',star_des[j],', ',star_des[k]);
writeln('The angular distance of these two stars is: ',ad*pi/0.180:6:1,' mrad');
writeln('                                  that  is: ',ad:6:2,' degrees');
writeln('                                  that  is: ',ad*60:8:1,' minutes');
writeln('                                  that  is: ',ad*3600:10:0,' seconds of arc');
writeln('The position angle of the second star is:   ',pa:6:2,' degrees');
     if not AllVisible then
      begin
       if des[j]<0 then writeln('The first star was below horizon.');
       if des[k]<0 then writeln('The second star was below horizon.');
      end;
    end
   else
    begin
     write(star_des[j],', ',star_des[k]);
     case in_units of
      'R': write(ad*pi/0.180:6:1,' mrad ');
      'D': write(ad:6:2,' degrees ');
      'M': write(ad*60:8:2,' minutes ');
     end;
     write(pa:6:2,' degrees');
     if not AllVisible then
      begin
       if des[j]<0 then write('- ') else write('  ');
       if des[k]<0 then write('-');
      end;
     writeln;
    end;
  end;

end.
