 {~ Semilog ~}

{$N+} {to enable >double< variable because of large exponents}
{$E+}   {to run on a computer without numeric coprocessor}

unit semilog; {from Faintness to brightness, by Jan Hollan, 1995}


interface

uses solar_ut;

const

LightSpeed=299792458.0;    {m/s : from definition of 1 m}
{AU=1.49597870E11 is defined in solar_ut}
pc2AU=206264.806; {1''/1 rad:  parsec to astronomical units}
tr_year=86400*365.242199; {s in 1900}
 
pc2m=AU*pc2AU {=3.0856776E16 parsec to meters};
ly2pc=LightSpeed*tr_year/pc2m {=0.9460528/3.0856776 light year to parsecs};

sr1pc=pc2m*pc2m; {m2: surface of 1 sr area of sphere of 1 pc radius}

                {0 mag levels of brightnesses, i.e. flux densities}
Mbol0=3.055E28;  {W, defined at IAU Gen Assembly in Kyoto, August 1997}
j0vis=2.54E-6;  {lm.m-2 : visual photopic}
j0V=3.2E-9;     {W.m-2  : V filter}  
j0B=6.5E-9;              {B filter}
j0U=2.8E-9;              {U filter}
{j0bol: see beginning of the programme}

var pi4,j0bol:real;

function Dn(n:byte;x:double):string;
 {semilogarithmic expression with n (4, e.g.) decimal places 
                      in the form #.dddd*10^##}

function DnE(n:byte;x:double):string;
 {semilogarithmic expression with n (4, e.g.) decimal places
                      in the form #.ddddE##}

function D2(x:double):string;
  {semilogarithmic expression with 2 decimal places in the form #.dd*10^##}

implementation

function Dn(n:byte;x:double):string;
 {semilogarithmic expression with n (4, e.g.) decimal places 
                      in the form #.dddd*10^##}
var fullexp,mantt:string; mant:real; j:integer;
begin
 str(x,fullexp);
 val(copy(fullexp,1,15),mant,j);
 str(mant:n+3:n,mantt);
 j:=pos('+',fullexp);
 if j=0 then j:=pos('-',fullexp);
 Dn:= mantt+'*10^'+copy(fullexp,j,1)+
                                +copy(fullexp,length(fullexp)-1,2)
end;

function DnE(n:byte;x:double):string;
 {semilogarithmic expression with n (4, e.g.) decimal places
                      in the form #.ddddE##}
var fullexp,mantt:string; mant:real; j:integer;
begin
 str(x,fullexp);
 val(copy(fullexp,1,15),mant,j);
 str(mant:n+3:n,mantt);
 j:=pos('+',fullexp);
 if j=0 then j:=pos('-',fullexp);
 if copy(fullexp,length(fullexp)-1,1)='0' then
  DnE:= mantt+'E'+copy(fullexp,j,1)+
                                +copy(fullexp,length(fullexp),1)
 else
  DnE:= mantt+'E'+copy(fullexp,j,1)+
                                +copy(fullexp,length(fullexp)-1,2);
end;

function D2(x:double):string;
  {semilogarithmic expression with 2 decimal places in the form #.dd*10^##}
begin
 D2:= Dn(2,x);
end;

begin
 pi4:=4*pi;
 j0bol:=Mbol0 / (100*sr1pc*pi4);  {bolometric, defined in Kyoto}
 {j0bol=2.55329E-8;}
end.
