 {~ Radiative balance  -- temperatures on insolated bodies ~} 

 { developed by Jan Hollan, when studying greenhouse effect on Mars and lunar
   temeratures}

var T,ST,dT,dTd,sinh,InSol,time_step,Tinf,Tb1,Tb2,Tb3,Sol,TR,TC3,SBCe:real;
 i,ii,Par_Count:integer;
 cycle:byte; j:word;
 Td: array[1..1000] of real; Tl:array[450 .. 4049] of real;
 par:string[20]; sw:char;

const
 SolRel:real=0.8;   {Maximum insolation Relative to "Solar constant"}
 time01d:real=24.0  {24.7 for Mars,} {709 for Moon};
 MaxDep:integer={1}98;     {Maximum  depth for diffusion / 1 cm}
 N_Cycles:integer=10;      {Number of Cycles to be computed}
 Comp_Diffu:boolean=true{false};    {Compute diffusion?}
 Write_Diffu:boolean=false;         {Write temperatures in various depths?}
 (* SolConst=1326.0; {W/m2}     {*0.433 for Mars}
 *)

 SolConst=1368.0{+-1.0}; {W/m2}     {according to K.J.Phillips,
                                   Guide to the Sun, pp. 315--317;
  the lowest extreme due to large spots has been 1365 W/m2
  according to SMM satellite measurements from 1978--1989}

 SBC=5.67E-8;     {constant of the Stefan-Boltzmann law}
 N_at_height:byte=1;

 A:real=1;
 E:real=1;
 TCS:real=960.0 {J/(kg*K) chosen specific thermal capacity of dry sand};
 Dv:real=1750.0 {kg/m3    chosen Volume density of sand};
 Cs:real=0.95 {W/(m*K)    chosen specific thermal conductivity of sand};

           {For Moon:}
 TC1:real=2.3E5 {2.0E6};   {J/K thermal capacity of superconducting 1 m2 block 1}
 TC2:real=1.7E5 {1.5E6};   {J/K thermal capacity of superconducting 1 m2 block 2}

 {just initialization:}
 STb1:real=0;
 STb2:real=0;
 STb3:real=0;

 cl=#13#10;

procedure help;
begin
 write(
'Radiative balance on insolated bodies. Parameters are:',cl,cl,

'S<maximum insolation relative to 1368 w/m2 (default is 0.8)>',cl,
'D<duration of day / 1 h (default 24, 24.7 would fit Mars, 709 Moon)',cl,
'TB1<thermal capacity of superconducting block with 1 m2 area / (J/K)>',cl,
'   (arbitrary, default=2.3E5, 2.0E6 would suit for Moon)',cl,
'TB1<thermal capacity of another superconducting block with 1 m2 area / (J/K)>',cl,
'   (arbitrary, default=1.7E5, 1.5E6 would suit for Moon to get similar',cl,
'   mean temperature as for thermal diffusion computation)',cl,
'A<absorptivity> (default 1 as for an absolutely black body.',cl,
'  For real surfaces, absorptivity < 1 and it depends on angle of incidence.)',cl,
'E<emissivity> (default 1 as well. Constant emissivity is an approximation,',cl,
'  as it depends on temperature of radiating surface a bit)',cl,
'Cn when no diffusion should be computed;',
'   when it should be computed, then:',cl,
' M<maximum depth / 1 cm  for computation, default 98>',cl,
' N<number of cycles (revolutions = days) to compute, default 10>',cl,
' W+ when temperatures at various depths should be written 6 times a day',cl,
' TS<specific thermal capacity of material / (J/(kg*K))> (default 960, for sand)',cl,
' Dv<volume density / (kg/m3)> (default 1750 as for sand)',cl,
' Cs<specific thermal conductivity / (W/(m*K))> (default 0.95)',cl,cl,

'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'
);
 halt;
end;

procedure Diffusion;
begin
    for j:=1 to MaxDep do
     begin
      dTd:=(Td[j]-Td[j+1])/TR;
      Td[j]:=Td[j]+time_step*(dT-dTd);
      dT:=dTd;
     end;
    Td[MaxDep+1]:=Td[MaxDep+1]+time_step*dt;
end;


begin
 Par_Count:=paramcount;
 if Par_Count>0 then for
  i:=1 to Par_Count do
   begin
    par:=paramstr(i);
    if (pos('?',par)>0) or (pos('-h',par)>0) then help;
    if (par[1]='-') or (par[1]='/') then par:=copy(par,2,length(par)-1);
    if length(par)>1 then
     begin
      sw:=UpCase(par[1]);
      par:=copy(par,2,length(par)-1);
      case sw of
       'A': begin
             val(par,A,ii);
             if ii<>0 then help;
            end;
       'C': if UpCase(par[1])='S' then
              begin
               if length(par)>1 then
                begin
                 val(copy(par,2,length(par)-1),Cs,ii);
                 if ii<>0 then help;
                end
              end
             else
            Comp_Diffu:=not((par[1]='-') or (UpCase(par[1])='N'));
       'D': begin
             if UpCase(par[1])='V' then
              begin
               if length(par)>1 then
                begin
                 val(copy(par,2,length(par)-1),Dv,ii);
                 if ii<>0 then help;
                end
              end
             else
             val(par,time01d,ii);
             if ii<>0 then help;
            end;
       'E': begin
             val(par,E,ii);
             if ii<>0 then help;
            end;
       'M': begin
             val(par,MaxDep,ii);
             if ii<>0 then help;
            end;
       'N': begin
             val(par,N_Cycles,ii);
             if ii<>0 then help;
            end;
       'T': if length(par)>1 then
             case UpCase(par[1]) of
             'S':
              begin
                begin
                 val(copy(par,2,length(par)-1),TCS,ii);
                 if ii<>0 then help;
                end
              end;
             'B':
              if length(par)>2 then
               case par[2] of
                '2': begin
                      val(copy(par,3,length(par)-2),TC2,ii);
                      if ii<>0 then help;
                     end;
               else  begin
                      val(copy(par,3,length(par)-2),TC1,ii);
                      if ii<>0 then help;
                     end;
               end;
              end;
       'S': begin
             val(par,SolRel,ii);
             if ii<>0 then help;
            end;
       'W': Write_Diffu:=not((par[1]='-') or (UpCase(par[1])='N'));
      end;
     end;
   end;
SBCe:=SBC*E;
Sol:=A*SolConst*SolRel;
Tinf:=sqrt(sqrt(Sol/(pi*SBCe)));
Tb1:=Tinf; Tb2:=Tinf; Tb3:=Tinf;
if time01d>48 then
 N_at_height:=round(time01d) div 24;
time_step:=time01d/N_at_height;
TC3:=0.01*TCS*Dv  {J/K  thermal capacity of 1 cm thick 1 m2 large layer:
     16800 J/K  for sand};
TR:=TC3/(CS/0.01) {J/W = s; 'time of temperature exchange' ---
                   the step of integration should be 5 times less at least};

writeln(
'Maximum absorbed solar input is ',Sol:6:1,' W/m2, or',A*SolRel:5:2,
                        ' of the so-called Solar constant of',cl,
'                       ',SolConst:6:1,' W/m2;',cl,
'the initial temperature for the moment of',cl,
'Sun being at quarter of his way across the sky',cl,
'is ',Tb1:6:1,' K',cl,
'(an equilibrium temperature for a layer of infinite thermal inertia)');



 ST:=0;
for j:=1 to MaxDep+1 do Td[j]:=Tb3;
for cycle:=1 to N_cycles do
 begin
  writeln(cl,'Day ',cycle,':',cl,cl,
'Angle from sunrise / 1 degree',cl,
'     Temperatures / 1 K');
  for i:=450 to 4049 do
   begin
    sinh:=sin(i*pi/1800);
    if sinh>0 then
     Insol:=Sol*sinh
    else
     Insol:=0;
    if cycle=1 then  {sqrt is slow sometimes, so just once}
      begin
       T:=sqrt(sqrt(Insol/SBCe));
       Tl[i]:=T;
      end
    else
     T:=Tl[i];
    ST:=ST+T;
    for ii:=1 to N_at_height do
     begin
      Tb1:=Tb1+time_step*(InSol - SBCe*sqr(sqr(Tb1)))/TC1;
      STb1:=STb1+Tb1;
      Tb2:=Tb2+time_step*(InSol - SBCe*sqr(sqr(Tb2)))/TC2;
      STb2:=STb2+Tb2;
      if Comp_Diffu then
       begin
        dT:=(Tb3-Td[1])/TR;
        Tb3:=Tb3+time_step*((InSol - SBCe*sqr(sqr(Tb3)))/TC3 - dT);
        Diffusion;
        STb3:=STb3+Tb3;
       end;
     end;
    if i mod 200 = 0 then
     begin
      write((i div 10) mod 360:4,T:6:1,Tb1:6:1,Tb2:6:1);
      if Comp_Diffu then
       begin
        writeln(Tb3:6:1);
        if Write_Diffu and (i mod 600 = 0) then
         for j:=1 to MaxDep do writeln('depth:',j:4,Td[j]:6:1);
       end
      else
       writeln;
     end;
   end;
  ST:=ST/3600.0;
  STb1:=STb1/(3600.0*N_at_height);
  STb2:=STb2/(3600.0*N_at_height);
  STb3:=STb3/(3600.0*N_at_height);
  write('mean',ST:6:1, STb1:6:1, STb2:6:1);
  if Comp_Diffu then write(STb3:6:1);
  writeln(  sqrt(sqrt(Sol/(pi*SBCe))):6:1,
          sqrt(sqrt(Sol/(4*SBCe))):6:1);
  write('       no ',TC1/1E6:5:2,TC2/1E6:6:2);
  if Comp_Diffu then write('  soil');
  writeln('  inf.  inf.sphere  are thermal capacities');
  writeln('          / (1 MJ/(m2*K))');
  writeln('       of some superconducting bodies');
  if Comp_Diffu then
   begin
    writeln('       And some       "soil"    or material with');
    writeln('TCS=',TCS:5:0,' J/(kg*K) specific thermal capacity');
    writeln('Dv =',Dv:5:0,' kg/m3  (volume) density');
    writeln('Cs =',Cs:5:3,' W/(m*K)  specific thermal conductivity');
   end;

  ST:=0;
  STb1:=0;
  STb2:=0;
  STb3:=0;
  if Comp_Diffu and (Write_Diffu or (cycle=N_cycles)) then
   begin
    for j:=1 to MaxDep do writeln('depth:',j:4,Td[j]:6:1);
    writeln(
  '          are temperatures at corresponding depths / 1 cm');
    writeln(
  '          when the Sun is at quarter of his way across the sky.');
   end;
 end;
end.
