 {~ Angular height of a point (or star in the Bright Star Catalogue) ~}

Program Star_Pos; {by Jan Hollan; may use B5_int.dbf}
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

uses DbBridge, angles_o, Un_Str_M, str_num, solar_ut, params, precesse,
 bsc_dir;      {just the loc_bsc constants}

const
h1=cl+
'Star_pos <point> dDD[.MM[.YY]] tHH[.MM[.SS]] UT# L# F# E# [<point2> <point3>]]'+cl+cl+

'gives angular height and azimuth (and hour angle and declination)'+cl+
' of the point.'+cl+cl;
h2=
'The point can be given through its HR number (in the Bright Star Catalogue)'+cl+
' as HR#,'+cl+
'by its Greek/Flamsteed code preceded by N as nAlp_Cas, nAlp1Sco, n42_Eri,'+cl;
h21=
'or by giving RA# (in degrees) or RAhh:mm:ss and'+cl+
'             De#   -  "  -    or DEdd:mm:"" .'+cl2+
'If more than one point is given, an output table is produced'+cl;
h3=
' D  is date (initial value is the current one)'+cl+
' T  is time (default is the current one)'+cl+
' UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)'+cl+
' L  is the Longitude of the observing place, default that of '
 +MyLocation_Name+','+cl;
h4=
' F  is the latitude      -               "                -  '+cl+
' E   equinox year (default 2000.0, i.e. J2000), for computing precession '+cl;
h6=
'( (C) Jan Hollan, N.Copernicus Observatory and Planetarium in Brno, 1999;'+cl+
' subject to the GNU General Public License, http://www.gnu.org/copyleft;'+cl+
' source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl;

BSC:boolean=false;
found:boolean=false;
part_given:boolean=false;
Greek_given:boolean=false;
PrecDate:real=2000.0;
ns: byte=1;
V: integer=-999;
B_V: integer=-999;
var
ParVal:string;
hr,hrhh,hrmm,hrss,tt: word;
hrdd,nflam:integer;
na,ndec,ParN,nname,nV,nB_V:byte;
sname,cname:string[3]; pname:char;
ras,des: array[1..20] of real;
Vs,B_Vs: array[1..20] of integer;
star_des: array[1..20] of string[15];
star_des_b: array[1..160] of byte absolute star_des;
code,j:integer;
full:boolean;

procedure Help(s:string);
begin
 if BSC then DbClose;
 if s<>'' then writeln(cl+s+cl);
 write(h1,h2,h21,h0enter);
 cont_or_halt;
 write(h0clear,cl,h3,h4,h_dir_bsc,cl,h_refraction,cl,h6);
 halt;
end;

procedure one_star;
begin
 ras[ns]:=ra;
 des[ns]:=decl;
 Vs[ns]:=V;
 B_Vs[ns]:=B_V;
 ra:=-999;
 decl:=-999;
 V:=-999;
 B_V:=-999;
 if star_des[ns]='' then
  star_des[ns]:='['+SRe(6,2,ra)+';'+SRe(6,2,decl)+']';
 inc(ns);
end;

begin
ra:=-999;
decl:=-999;
FillChar(star_des_b[1],56,0);
if paramcount=0 then help('')
else
 for ParN:= 1 to paramcount do
  begin
   Par:=ParamStr(ParN);
   if Par[1] in ['/','-'] then Par:=copy(Par,2,length(par)-1);
   if length(par)>1 then ParVal:=copy(par,2,length(par)-1);
   Par:=__cvtstr(Par,_To_UpCase_Str);
   case Par[1] of
    '?','-': Help(''); {- added to respect unix --help even without the help}
    'H','N':
        if length(par)>1 then
         begin
          if not BSC then
           if not DbInitSearch(Loc_bsc,'b5_int.dbf') then
            help(
'The Bright Star Catalogue not found in current directory or directories:'+cl+
Loc_bsc)
           else
            begin
             BSC:=true;
             na:= DbFieldNum('rh2');
             ndec:=DbFieldNum('dd2');
             nname:=DbFieldNum('name');
             nV:= DbFieldNum('V');
             nB_V:=DbFieldNum('B_V');
            end;
          hr:=1;
          if par[1]='H' then
           begin
            hr:=round(Par_num(copy(Par,2,length(par)-1)));
            if not DbRead(hr,rec) then
             help('There is no such number in the Bright Star Catalogue: '
                   +SI(4,HR));
           end
          else if length(par)>6 then
           begin
            found:=false;
            if pos('_',ParVal)>0 then
             begin
              sname:=ItemStrD('_',1,ParVal);
              nflam:=I_S(sname);
              Greek_given:= nflam=0;
              cname:=ItemStrD('_',2,ParVal);
              pname:='_';
             end
            else
             begin
              sname:=copy(ParVal,1,3);
              cname:=copy(ParVal,5,3);
              pname:=ParVal[4];
              part_given:=true;
             end;
            hr:=1;
            if not DbRead(hr,rec) then;
            repeat
             if copy (rec[nname],8,3)=cname then
              if Greek_given then
               begin
                found:=copy(rec[nname],4,3)=sname;
                if part_given then found:= found and (rec[7]=pname);
               end
              else
               begin
                found:=I_S(copy(rec[nname],1,3))=nflam;
                if part_given then found:= found and (rec[7]=pname);
               end;
             inc(hr);
            until found or not DbRead(hr,rec);
            if not found then help(+sname+pname+cname+'  not found');
           end;
          DbDisp(hr);
          writeln;
          val(__cvtstr(rec[na],_rem_white_str),hrhh,j);
          val(__cvtstr(rec[na+1],_rem_white_str),hrmm,j);
          val(__cvtstr(rec[na+2],_rem_white_str),hrss,j);
          tt:=hrss-(hrss div 10)*10;
          hrss:=hrss div 10;
          if Bab2Dec(1,hrhh,hrmm,hrss,tt,ra,'H') then;
          ra:=ra*15;
          val(__cvtstr(rec[ndec],_rem_white_str),hrdd,j);
          val(__cvtstr(rec[ndec+1],_rem_white_str),hrmm,j);
          val(__cvtstr(rec[ndec+2],_rem_white_str),hrss,j);
          tt:=hrss-(hrss div 10)*10;
          hrss:=hrss div 10;
          if pos('-',rec[ndec])>0 then
           begin
            hrdd:=abs(hrdd);
            if Bab2Dec(-1,hrdd,hrmm,hrss,tt,decl,'D') then
           end
          else
            if Bab2Dec(1,hrdd,hrmm,hrss,tt,decl,'D') then;
          star_des[ns]:=par;
          val(__cvtstr(rec[nV],_rem_white_str),V,j);
          val(__cvtstr(rec[nB_V],_rem_white_str),B_V,j);
          one_star;
         end
        else help('');
    'R': if length(par)>2 then
          begin
           if par[2]='A' then
            begin
             par:=copy(par,3,length(par)-2);
             val(par,ra,code);
             if code > 0 then
              if BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'H') then
               ra:=ra*15
              else
               if not BabS2Dec(Par,hrhh,hrmm,hrss,tt,ra,'D') then
                help(
   'RA gives no valid right ascension (either degrees or hh:mm:ss): '+par);
             if decl<>-999 then one_star;
            end;
           if par[2]='H' then
            begin
             Relative_Humidity:=ss2r(par,3);
             if Relative_Humidity<0.01 then
              help('RH gives no valid relative humidity: '+par);
            end;
          end
         else
          help(par+' gives no value!');
    'D': if length(par)>1 then
          if par[2]='E' then
           begin
            if length(par)>2 then
             begin
              par:=copy(par,3,length(par)-2);
              val(par,decl,code);
              if code > 0 then
               if BabS2Dec(Par,hrhh,hrmm,hrss,tt,decl,'D') then
               else
                help(
   'De gives no valid declination (either degrees or dd:mm:ss): '+par);
             end
            else
                help(
   'De gives no declination (either degrees or dd:mm:ss): '+par);
            if ra<>-999 then one_star;
           end
          else if not Par_Date then help('');
   'P': if length(par)>2 then
          case par[2] of
          'R':
            begin
             pressure_db_101325:=ss2r(par,3);
             if pressure_db_101325<0.02 then
              help('PR gives no valid pressure divided by 1013.25 Pa: '+par);
            end
          else
           help(par+' is no known parameter!')
          end
        else
         help(par+' gives no value!');
   'C': if length(par)>2 then
          if par[2]='T' then
           begin
            Celsius_temp:=ss2r(par,3);
            if Celsius_temp<-273 then
             help('CT gives no valid Celsius temperature / 1 K: '+par);
           end
          else
           help(par+' is no known parameter!')
        else
         help(par+' gives no value!');
   'E': PrecDate:=ss2r(par,3);
   'F': if not Par_Fi then help('');
   'L': if length(Par)>2 then
         case  Par[2] of
          'B': Loc_BSC:=copy(Par,3,length(par)-2);
          {'E': Loc_obj:=copy(Par,3,length(par)-2);}
         else
          if not Par_Lambda then help(' ');
         end
        else
         if not Par_Lambda then help(' ');
   'T': if not Par_Time then help('');
   'U': if not Par_UT then help('');
   else help('');
   end;
  end;

if BSC then DbClose;

dec(ns);
if ns<1 then help('Nothing to do - at least one point needed to make anything');
if ns>20 then help('Sorry, 20 stars is the maximum');

Get_JD;
if PrecInit(PrecDate,YYYYreal(JD)) then;
full:=ns=1;
if not full then
 writeln(
         '#  V  ',' B-V ','   h   ','         ',' azimuth',cl,
         '# ----',' ----',' ------',' Air mass',' -------','  designation',cl,
         '# mmag',' mmag',' degree','         ','  degree',cl);
for j:=1 to ns do
 begin
  Precess(RAs[j],Des[j]);
  A_h_PA(SM,RAs[j],Des[j],A,h,PA,Air_mass);
  if h>-0.8 then
   begin
    {h:=refract_d(h,0.97,10,70);}
    h:=refract_d(h,pressure_db_101325,Celsius_temp,Relative_humidity);
    if full then
     writeln('The (refracted) angular height of the point is: ',h:6:2,' degrees');
   end
  else
   if full then
    writeln(
  'The angular height of the point is: ',h:6:2,' degrees');
   if full then
    begin
     writeln('and azimuth:                                    ',
        Bas_Int(a-180,360):6:2,' degrees');
     writeln(
  '(from the south, i.e., "astronomical" azimuth:  ',a:6:2,' degrees)');
     if h>=0 then
      writeln(cl,
  'Air mass (mass of a column of air in this direction',cl,
  'relative to the mass of the vertical column of air) is: ',Air_mass:6:2);
     a:=deg_to_rad(a); h:=deg_to_rad(h);
     ah_to_td(-(pi/2-fi),a,h);
     a:=rad_to_deg(a);
     writeln(cl,
     'Hour angle is',a:7:2,' degrees ( = ',h_min_s(a/15),'  in the hh:mm:ss system),',cl,
     'declination is',rad_to_deg(h):6:2,' degrees.');
     writeln(cl,
     'Right ascension is',Bas_int(SM*15-a,360):7:2,' degrees.');
    end
   else
    writeln(Vs[j]:5,B_Vs[j]:6,h:7:2,Air_mass:9:2,a:8:2,'  ',star_des[j]);
 end;

end.
