 {~ Equinoxes and solstices ~}

Program sun_ph; {by Jan Hollan, using a unit by D.Motl}
(*  Copyright (C) 1999 Jan Hollan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

uses {J.H.'s units:}
     solar_ut,angles_o,params,str_num,
     planets {co-written by David Motl},
     Un_Str_M {instead of Unit_Str by Blaise Computing, Inc.} ;
const
 Next:word = 0;
 Step:byte = 1;
 NeededMonths:set of byte = [3,6,9,12];
 NrM:byte=4; {Number of needed months}
cl=#13#10;
h1=cl+
'A program from Jan Hollan, N. Copernicus Observatory and Planetarium in Brno,'+cl+
' (subject to the GNU General Public License, http://www.gnu.org/copyleft,'+cl+
'  source code available at http://astro.sci.muni.cz/pub/hollan/programmes)'+cl
  +cl;
h1a=
'gives equinoxes and solstices accurate to some 5 minutes:'+cl+
   +cl+
'Sun_ph Y#### UT#   S# N# NM#:#:#   TT#[unit]'+cl+cl;
h2=
'Y  is the year (default that given by the computer clock)'+cl+
'UT is the t-UTC difference / 1 h (default 1 from Nov. to March, else 2)'+cl2;
h3=
' S   step to the next wanted year / 1 a  (default 1)'+cl+
' N   number of such next steps (default 0)'+cl+
' NM  needed months given as #[:#[:#]] (default all of them, i.e., 3:6:9:12)'+cl2;
NDn:integer=19;
OnceMore:boolean=true;

var
 Name:string;
 l,b,r,ls,lg_old,lg_new: real;
 i: byte;
 last_phase :shortint;
 code: integer;
 j : integer;

procedure Help;
begin
 write(h1,h1a,h2,h3,hTT); halt
end;

begin
  {Datum:=__DateStr(YYYY,MM,DD);
   t_UT:=1;}      {difference common time - Universal Time}

  if paramcount>0 then for j:= 1 to paramcount do
   begin
    Par:=paramstr(j);
    Par:=__cvtstr(Par,_To_UpCase_Str);
    if (Par[1]='/') or (Par[1]='-') then
      Par:=copy(Par,2,length(par)-1);
    case Par[1] of
     '?','-': Help; {- added to respect unix --help even without the help}
     'T' : if not Par_tt_ut then help;
     'Y' :begin
          Par:=copy (Par,2,length(Par)-1);
          val(Par,YYYY,code);
          if code>0 then
           begin
            writeln('The Year parameter is no number');
            help
           end
          end;
      'U': if not Par_UT then help;
      'N': if length(Par)>1 then
            if Par[2]='M' then
             begin
              if length(Par)>2 then
               begin
                NeededMonths:=[];
                Par:=copy(Par,3,length(Par)-2);
                NrM:=ItemCountD(':',Par);
                for i:= 1 to NrM do
                 NeededMonths:=NeededMonths+[I_S(ItemStrD(':',i,Par))];
               end
             end
            else
             Next:=par2i(Par);
      'S': Step:=par2i(Par);
    end; {of case}
   end; {of for}

  UT:= 0 - t_UT    {to avoid wrong date at midnight:}
                   + 1E-5;
  MM:=3;
  UT:=Bas_int(UT,24);

  writeln(cl,
'Mean (i.e., like if there were no nutation of axis of rotation of the Earth)',cl,
'Equinoxes and Solstices expressed in time which is ',t_UT:4:1,
' h ahead of the UTC:',cl);

if not tt_UT_set then {just to prepare the output sentence on the TT - UT}
 begin
  ND:=NDn;
  if UT<0  then dec(ND);
  if UT>24 then inc(ND);
  Sid_time_loc(SM,JD,UT,Lambda);
  tt_ut:=tt_UTC_in_days(JD);
 end;
if tt_UT_set then
  writeln('Terrestric Time - UT set to: ',tt_UT*24:6:2,' h',cl)
else if tt_UT>0.001 then
  writeln('Terrestric Time - UT assumed initially: ',tt_UT*24:6:2,' h',cl);

for j:=1 to Next+1 do
begin

 repeat

    repeat
     ND:=NDn;
     if UT<0  then dec(ND);
     if UT>24 then inc(ND);
     Sid_time_loc(SM,JD,UT,Lambda);
     if not tt_UT_set then tt_ut:=tt_UTC_in_days(JD);
     Planet_hel(3,jd+tt_ut,l,b,r,Name);
     ls:=bas_int(l+180,360);
     last_phase:= trunc(ls/90) mod 4;
     if OnceMore then
      if ls<90 then        {to catch the moment even in Julian calendar}
       Ndn:=NDn-trunc(ls)-1
      else
       OnceMore:=false;
    until not OnceMore;

    repeat
     JD:=JD+1;
     Planet_hel(3,jd+tt_ut,l,b,r,Name);
     ls:=bas_int(l+180,360);
     Name:=DoW(JD,t_UT,_English);
    until
     (trunc(ls/90) > last_phase)
     or ((last_phase=3) and (trunc(ls/90)=0));

    lg_new:=ls;
    inc(last_phase);
    last_phase:=last_phase mod 4;
    JD:=JD-1;
    Planet_hel(3,jd+tt_ut,l,b,r,Name);
    ls:=bas_int(l+180,360);
    Name:=DoW(JD,t_UT,_English);
    lg_old:=ls;
    if MM in NeededMonths then
     begin
      write(yyyy:4,'-',mm:2,'-',nd:2,'  ',Name);
      writeln(24*Bas_int(last_phase*90-lg_old,360)
                /Bas_int(lg_new-lg_old,360):5:1,' h');
     end;

  inc(MM,3);

 until MM>12;

 if (NrM>1) or (j mod 5 = 0) then writeln;

 inc(YYYY,step);
 MM:=3;

end;

end.
