 {~ Reading, writing into and creating dBaseIII+ files ~}
 {~ (based on an old programme by Milan Drasil). ~}

Unit DbBridge;
     {written by Milan Dr\'a\v sil, slightly enriched by Jan Hollan}
(*  Copyright (C) 1999 Jan Hollan;
  by "program" this unit is further meant.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

Interface
uses Dos;

const
 max_fields=60;     {Maximum allowed number of fields}
 max_length=2048;   {Maximum allowed length of a record}

type inforec = record
                field : string[10];
                typ   : char;
                len   : byte;
                dec   : byte;
               end;

     DbStr   = array[1..max_fields] of string;


var DbInfo    : array[1..max_fields] of inforec; {field description}
    DbInfoS   : array[1..max_fields] of inforec; {field description entered
                                           to structure description}
    DbRecl    : word;                    {record length}
    DbOff     : word;                    {Start of records}
    DbNuRec   : longint;                 {Number of Records}
    DbNuRecS  : longint;                 {Number of Records of Structure desc.}
    DbFileName: PathStr;                 {Name of File, incl. path}
    rec : DbStr;                         {one record - loose}
    OneRec : array [1..max_length] of char;    {one record - dense}
    fd   : file ;                {to have a possibility to access the file  }
const
    DbOK : boolean=true;              {              directly   - not recommended}
    DbNuField : word=0;                    {Number of Fields}


Function  DbInit(f : string) : Boolean;
           {opening an existing file - False if not found}
function  DbInitSearch(StandLoc:string;SomeFileName:PathStr):boolean;
           {adding suffix .dbf if needed and then, if no directory is given
            in DbFileName and file is not found in the current one, searching
            through the StandLoc (DirString: series of directories
            delimited by ;).
            If still not found (or DbInit fails),
            writing a message on the screen.}
Function  DbRead(RecNo : longint; var rec : DbStr) : boolean;
           {reading a record number RecNo}
Function  DbFieldNum(F_Nam:string):byte;
           {gives the number of the field F_Nam; 0 when fails}

Procedure DbDisp(recno:longint);
           {displays a record number RecNo on the screen}

Procedure DbNewField(F_Nam:string;F_Typ:char;F_len,F_dec:byte);
           {adding a field to a file to be created}
Function  DbCreate(f: string): Boolean;
           {creating a file with fields defined by the above procedure}
Function  DbNewFrom(f : string): Boolean; {creating a file according to
            the structure created by DbStruDesc,DbDescField,DbClose}
Procedure DbWrNum(FieldNum: byte; Value: real);
           {filling the corresponding field with a number}
Function  DbWrite(recno : longint; var rec : DbStr) : boolean;
           {writing a record number RecNo;
                                    RecNo=0 asks for appending a new record}
Procedure DbStruDescr(f:string);
           {creates a file describing structure of another dBase file}
Procedure DbDescField(F_Nam:string;F_Typ:char;F_len,F_dec:byte;
                       f_desc,f_comments:string);
           {adds a record to this file}

Procedure DbClose;
           {closing the file and setting the current date, if needed}


Implementation

type hearec = record
               DbSign,YY,MM,DD   :byte;
               reccount : longint;
               offset   : word;
               reclen   : word;
               n3       : array [1..5] of longint;
              end;
type idrec  = record
               field: array[1..10] of char;
               n1   : byte;
               typ  : char;
               n2   : longint;
               len  : byte;
               dec  : byte;
               n3   : array [1..14] of byte
              end;


var i,j : byte;
    v1      : hearec;
    f1      : file of hearec;
    v2      : idrec;
    f2      : file of idrec;

const
  DbAppend:boolean=false;
  DbOpen:boolean=false;


Function DbInit(f : string) : Boolean;

begin
 DbInit:=false;
 if DbOpen then Exit;
 if pos('.',f)=0 then f:=f+'.dbf';
 DbFileName:=f;

{$I-}
 assign(f1,f);
 FileMode:=0; {will open the files by reset as Read Only}
 reset(f1);
 if ioresult<>0 then
  begin
   {$I+}
   Exit;
  end;
{$I+}
 read(f1,v1);
 DbRecl:=v1.reclen;
 DbOff:=v1.offset;
 DbNuRec:=v1.reccount;
 DbNuField:=(DbOff div 32)-1;
 close(f1);
 if DbNuField>max_fields then
  Exit;

 assign(f2,f);
 reset(f2);
 seek(f2,1);
 for i:= 1 to DbNuField do
  begin
   read(f2,v2);
   DbInfo[i].field:='';
   j:=1;
   while (j<=10) and (v2.field[j]<>chr(0)) do
    begin
     DbInfo[i].field:=DbInfo[i].field+v2.field[j];
     inc(j);
    end;
   DbInfo[i].typ  :=v2.typ;
   DbInfo[i].len  :=v2.len;
   DbInfo[i].dec  :=v2.dec;
   rec[i][0]:=chr(DbInfo[i].len);
  end;
 close(f2);

 assign(fd,f);
 reset(fd,1);
 DbInit:=true;
 DbAppend:=false;
 DbOpen:=true
end;

function DbInitSearch(StandLoc:string;SomeFileName:PathStr):boolean;
 {adding suffix .dbf if needed and then, if no directory is given
  in DbFileName and file is not found in the current one, searching
  through the StandLoc (DirString: series of directories delimited by ;).
  If still not found (or DbInit fails), writing a message on the screen.}
var FoundName: PathStr; FDir: DirStr; FName: NameStr; FExt: ExtStr;
begin
 DbInitSearch:=false;
 FSplit(SomeFileName,FDir,FName,FExt);
 if FExt='' then SomeFileName:=FDir+FName+'.dbf';
 if not DbInit(SomeFileName) then
  begin
   if FDir='' then
   FoundName:=FSearch(DbFileName,StandLoc);
   if FoundName='' then
    begin
     writeln('dBase file ',DbFileName,' not found');
     exit
    end
   else
    if not DbInit(FoundName) then
     begin
      writeln('the file',FoundName,' seems to be not a dBase one.');
      exit
     end
  end;
 DbInitSearch:=true;
end;

Function DbCreate(f : string) : Boolean;
var Year,Month,Day,DoW:word;
const CR:byte=13;
begin
if (not DbOpen)
and DbOK {this is false if DbInfo field was not filled OK by DbNewFileld}
then
 begin
  if pos('.',f)=0 then f:=f+'.dbf';
  DbFileName:=f;
  v1.DbSign:=3;
  GetDate(Year,Month,Day,DoW);
  v1.YY:=Year-1900;
  v1.MM:=Month;
  v1.DD:=Day;
  FillChar(v1.n3,20,0);
  DbRecl:=1;
  for i:=1 to DbNuField do inc(DbRecl,DbInfo[i].len);
  v1.reclen:=DbRecl;
  DbOff:=(DbNuField+1)*32+1;
  v1.offset:=DbOff;
  v1.reccount:=0;
  DbNuRec:=0;
  assign(f1,f);
  rewrite(f1);
  write(f1,v1);
  close(f1);

  v2.n1:=0;
  v2.n2:=0;
  FillChar(v2.n3,14,0);
  assign(f2,f);
  FileMode:=2; {needs to open the file as writable}
  reset(f2);
  seek(f2,1);
  for i:= 1 to DbNuField do
   begin
    fillchar(v2.field,10,0);
    for j:=1 to length(DbInfo[i].field) do v2.field[j]:=DbInfo[i].field[j];
    v2.typ:=DbInfo[i].typ;
    v2.len:=DbInfo[i].len;
    v2.dec:=DbInfo[i].dec;
    write(f2,v2);
   end;
  close(f2);

  assign(fd,f);
  reset(fd,1);
  seek(fd,DbOff-1);
  BlockWrite (fd,CR,1);
  DbAppend:=false;
  DbOpen:=true;
  DbCreate:=true
 end
else
 DbCreate:=false
end;

Procedure DbNewField(F_Nam:string;F_Typ:char;F_len,F_dec:byte);
begin
If not (F_len>0) then DbOK:= False;
F_Typ:=UpCase(F_Typ);
case F_Typ of
  'C': F_dec:=0;
  'L': begin F_Len:=1; F_Dec:=0 end;
  'N': begin if (F_Len>19) or (F_dec>17) then DbOK:=False end;
  'D': begin F_len:=8; F_Dec:=0 end;
 {'I': begin if (F_Len>4) or (F_dec<>0) then DbOK:=False end;
  'R': begin if (F_Len>8) or (F_Len<4) or (F_dec<>0) then DbOK:=False end;
 }
 else DbOK:=False;
end;
if DbOK then
 begin
  inc(DbNuField);
  with DbInfo[DbNuField] do
     begin
      if length(F_Nam)<10 then i:= length(F_Nam) else i:=10;
      field:='';
      for j:=1 to i do field:=field+UpCase(F_Nam[j]);
      typ:=F_Typ; len:=F_len; dec:=F_dec
     end;
 end
end;

Function DbNewFrom(f : string): Boolean;
begin
 DbNewFrom:=false;
 if DbNuRecS>0 then
  begin
   for i:=1 to DbNuRecS do
    begin
     DbInfo[i].field:= DbInfoS[i].field;
     DbInfo[i].typ := DbInfoS[i].typ;
     DbInfo[i].len  := DbInfoS[i].len;
     DbInfo[i].dec  := DbInfoS[i].dec;
    end;
   DbNuField:=DbNuRecS;
   DbNewFrom:=DbCreate(f);
  end;
end;


Function DbRead(recno : longint; var rec : DbStr) : boolean;

var k: word;

begin
 if (recno>DbNurec) or (recno<1)
  then
   DbRead:=FALSE
  else
   begin
    DbRead:=TRUE;
    seek(fd,DbOff+(recno-1)*DbRecl);
    BlockRead (fd,OneRec,DbRecl);
    k:=2;
    for i:=1 to DbNuField do
     begin
      move(Onerec[k],rec[i][1],DbInfo[i].len);
      inc(k,DbInfo[i].len);
     {for j:=1 to DbInfo[i].len do
       begin
        rec[i][j]:=OneRec[k];
        inc(k);
       end;
      }
      {rec[i][0]:=chr(DbInfo[i].len); -- this is set in DbInit}
     end;
   end;
end;

Procedure DbWrNum(FieldNum: byte;Value:real);
begin
 str(Value:DbInfo[FieldNum].len:DbInfo[FieldNum].dec,rec[FieldNum]);
end;

Function DbWrite(recno : longint; var rec : DbStr) : boolean;

var k : word;

begin
 if recno>DbNurec then
  DbWrite:=FALSE
 else
  begin
    DbWrite:=TRUE;
    if recno=0 then
     begin
      if FileMode=0 then
       begin
        close(fd);
        FileMode:=2; {enables writing into the file}
        reset(fd,1);
       end;
      inc(DbNurec);
      seek(fd,DbOff+(DbNurec-1)*DbRecl);
      DbAppend:=true
     end
    else
     seek(fd,DbOff+(recno-1)*DbRecl);
    OneRec[1]:=' ';
    k:=2;
    for i:=1 to DbNuField do
     if length(rec[i])<DbInfo[i].len then   {If the string was shortened,}
       begin
        for j:=1 to length(rec[i]) do
         begin
          OneRec[k]:=rec[i][j];
          inc(k);
         end;                  {rest of the field is filled with blanks.}
        for j:=length(rec[i])+1 to DbInfo[i].len do
         begin
          OneRec[k]:=' ';
          inc(k);
         end
       end
     else {the length is the same (or greater:extra characters are ignored)}
       for j:=1 to DbInfo[i].len do
        begin
         OneRec[k]:=rec[i][j];
         inc(k);
        end;
    BlockWrite(fd,OneRec,DbRecl);
  end
end;

Function DbFieldNum(F_Nam:string):byte;
                       {gives the number of the field aux; 0 when fails}
var aux: string[10];
begin
  if length(F_Nam)<10 then i:= length(F_Nam) else i:=10;
  aux:='';
  for j:=1 to i do aux:=aux+UpCase(F_Nam[j]);
  i:=0;
  for j:= 1 to DbNuField do
    if DbInfo[j].field = aux then i:=j;
  DbFieldNum:=i
end;

Procedure DbStruDescr(f:string);
           {creates a file describing structure of another dBase file}
begin
 DbNewField('FIELD_NAME','C',10,0);
 DbNewField('FIELD_TYPE','C',1,0);
 DbNewField('FIELD_LEN', 'N',3,0);
 DbNewField('FIELD_DEC', 'N',3,0);
 DbNewField('DESCRIPTIO','C',32,0);
 DbNewField('COMMENTS',  'C',254,0);
 DbNuRecS:=0;
 if DbCreate(f) then;
end;


Procedure DbDescField(F_Nam:string;F_Typ:char;F_len,F_dec:byte;F_desc,F_comments:string);
           {adds a record to this file}
var fnam:string[10];
begin
If not (F_len>0) then DbOK:= False;
F_Typ:=UpCase(F_Typ);
If F_Typ='C' then F_dec:=0
else if F_Typ='L' then begin F_Len:=1; F_Dec:=0 end
     else if F_Typ='N' then
            begin if (F_Len>19) or (F_dec>17) then DbOK:=False end
          else if F_Typ='D' then begin F_len:=8; F_Dec:=0 end
               else DbOK:=False;
if DbOK then
 begin
  if length(F_Nam)<10 then i:= length(F_Nam) else i:=10;
  fnam:='';
  for j:=1 to i do fnam:=fnam+UpCase(F_Nam[j]);
  rec[1]:=fnam;
  rec[2]:=F_Typ;
  str(F_len:3,rec[3]);
  str(F_dec:3,rec[4]);
  rec[5]:=F_desc;
  rec[6]:=F_comments;
  if DbWrite(0,rec) then
   with DbInfoS[DbNuRec] do
    begin
     DbNuRecS:=DbNuRec;
     field:=fnam; typ:=F_Typ; len:=F_len; dec:=F_dec
    end;
 end;
end;

procedure DbDisp(recno:longint); {displays the current record on the screen}
begin
writeln('The whole record No.',recno,' is');
for j:=1 to DbNuField do
 begin
  write(DbInfo[j].field,' ');
  if length(DbInfo[j].field)<DbInfo[j].len then
   for i:=1 to DbInfo[j].len-length(DbInfo[j].field) do
    write(' ')
 end;
writeln;
for j:=1 to DbNuField do
 begin
  write(rec[j],' ');
  if length(DbInfo[j].field)>DbInfo[j].len then
   for i:=1 to -1*(DbInfo[j].len-length(DbInfo[j].field)) do
    write(' ')
 end;
writeln;
end;


Procedure DbClose;
var Year,Month,Day,DoW:word; eofchr: byte;
begin
if DbOpen then
 if DbAppend then
  begin
   seek(fd,DbOff+(DbNurec)*DbRecl);
   eofchr:=26;
   BlockWrite(fd,eofchr,1);
   close(fd);
   assign(f1,DbFileName);
   reset(f1);
   read(f1,v1);
   GetDate(Year,Month,Day,DoW);
   v1.YY:=Year-1900;
   v1.MM:=Month;
   v1.DD:=Day;
   v1.reccount:=DbNuRec;
   seek(f1,0);
   write(f1,v1);
   close(f1)
  end
 else
  close(fd);
FileMode:=0; {sets the file access by reset to Read Only again}
DbNuField:=0;
DbOK:=true;
DbOpen:=false;
end;

end.
